# -*- coding: utf-8 -*-
# Copyright (c) 2022, KarjaKAK
# All rights reserved.

import tkinter as tk
from tkinter import messagebox
from matplotlib.backends.backend_tkagg import (
    FigureCanvasTkAgg, NavigationToolbar2Tk
)
from matplotlib.figure import Figure
from dataclasses import dataclass
from types import MappingProxyType as mpt
from decimal import Decimal


__all__ = ["Charts"]


@dataclass(frozen=True, slots=True)
class Charts:
    """Creating charts for data given"""

    data: dict
    name: str
    colors: tuple

    def __post_init__(self):
        match (
            isinstance(self.data, dict), 
            isinstance(self.name, str),
            isinstance(self.colors, tuple)
        ):
            case (False, _, _):
                raise TypeError(f"{self.data!r} Excpected Dict Type!")
            case (_, False, _):
                raise TypeError(f"{self.name!r} Expected String Type!")
            case (_, _, False):
                raise TypeError(f"{self.colors} Expected Tuple Type!")
            case _:
                super(Charts, self).__setattr__("data", mpt(self.data))
                
    def pchart(self, root):
        try:
            root.title(self.name)
            frameChartsLT = tk.Frame(root)
            frameChartsLT.pack(fill = tk.BOTH, expand = 1)
            fig = Figure()
            wedge_properties = {"edgecolor": "k",'linewidth': 1, 'width': 0.3}
            explode = [0.05 for _  in range(len(self.data))]

            ax = fig.add_subplot()

            def make_autopct(values):
                def my_autopct(pct):
                    total = sum(values)
                    val = Decimal(pct)*total/100
                    return f"{pct:.2f}%\n{val:,.2f}"
                return my_autopct

            datval = [abs(n) for n in self.data.values()]
            ax.pie(
                datval, 
                radius=1.3, 
                explode=explode, 
                labels=self.data.keys(),
                colors=self.colors, 
                autopct=make_autopct(datval), 
                shadow=False, 
                wedgeprops = wedge_properties,
            )
            del datval

            chart1 = FigureCanvasTkAgg(fig,frameChartsLT)
            NavigationToolbar2Tk(chart1, frameChartsLT)
            chart1.get_tk_widget().pack(fill = tk.BOTH, expand = 1)
            tx = f"TOTAL {self.name.upper()}: {sum(self.data.values()):,.2f}"
            lab = tk.Label(frameChartsLT, text=tx, justify=tk.CENTER, bg="white", font="verdana 20 bold", fg = "black")
            lab.pack(fill=tk.X, ipady=2)
        except:
            messagebox.showerror("Charts", "Please check data or colors for validation!", parent=frameChartsLT)



# -*- coding: utf-8 -*-
# Copyright (c) 2022, KarjaKAK
# All rights reserved.

from dataclasses import dataclass
from types import MappingProxyType as mpt


__all__ = ["EvalExp"]


@dataclass(frozen=True, slots=True)
class EvalExp:
    """Class eval that controled what can be express"""

    expression: str
    _all: dict | None

    def __post_init__(self):

        match (
            isinstance(self.expression, str),
            isinstance(self._all, dict|None)
        ):
            case (False, _):
                raise TypeError(f"{self.expression!r} Expected String type!")
            case (_, False):
                raise TypeError(f"{self._all!r} Expected Dict or None!")
            case _:
                if self._all is not None: 
                    super(EvalExp, self).__setattr__("_all", mpt(self._all))

    def evlex(self):
        """Expression that controled by what allowed"""

        comp = compile(self.expression, "<string>", "eval",)
        try:
            for name in comp.co_names:
                if name not in self._all:
                    raise NameError(f"{name!r} is not allowed for expression!")
            else:
                return eval(self.expression, {"__builtins__": None}, self._all)
        except:
            raise
        finally:
            del comp



# -*- coding: utf-8 -*-
# Copyright (c) 2022, KarjaKAK
# All rights reserved.

import re
import os
from pathlib import Path
from itertools import islice
from dataclasses import dataclass, field
from treeview import TreeView
from decimal import Decimal


__all__ = ["SumAll"]


@dataclass(frozen=True, slots=True)
class SumAll(TreeView):
    """Sum all the string that converted to int or float"""

    filename: str 
    sig: str = field(kw_only= True)

    def __post_init__(self):
        if not (filename := self.findext(self.filename)):
            raise FileExistsError(f"This {filename!r} name is not exist!")    
        if not isinstance(self.sig, str):
            raise TypeError(f"Sign need to be a string type!")
        super(TreeView, self).__setattr__("filename", filename)

    def __len__(self) -> int:
        """Get the total number of indexes"""
        
        return len(self.getidx())

    @staticmethod
    def findext(filename) -> str | None:
        """Checker file path existence with TreeView format file"""

        match Path(filename):
            case pt if pt.exists() and ".txt" in pt.name:
                return str(pt.absolute())[:-4]
            case pt if os.path.exists(f"{pt.absolute()}.txt"):
                return str(pt.absolute())
            case _:
                return None

 
    def getsums(self, dt: bool = True) -> dict:
        """Sum-up every sub-node in the list"""

        lid = {}
        dgt = re.compile(r"\-?\d+\.?\d+")
        for id in self.getidx(dt):
            addnum = 0
            for i, t in islice(self.getdata(), id+1, None):
                if not t.startswith("\n"):
                    match (dt, 
                            s:= dgt.match(
                                t.strip()
                                    .rpartition(" ")[2]
                                    .replace(",", "")
                            ),
                            t.strip(" ").startswith("-TOTAL")
                        ):
                        case (True, s , False) if bool(s):
                            s = Decimal(s.group())
                        case (False, s, True) if bool(s):
                            s = Decimal(s.group())
                        case _:
                            s = 0
                    addnum += s
                else:
                    lid[i] = f"{addnum:,.2f}" if dt else addnum
                    break
            else:
                if i not in lid:
                    lid[i+1] = f"{addnum:,.2f}" if dt else addnum
        if dt: 
            return lid
        else:
            return lid.values()

    def getidx(self, dt: bool = True) -> list:
        """Getting the indicies of every node """

        idx = []
        for i, t in self.getdata():
            match (dt, t.startswith(self.sig), t.strip(" ").startswith("-TOTAL")):
                case (True, True, False):
                    idx.append(i)
                case (True, False, True):
                    idx.pop()
                case (False, True, _):
                    idx.append(i)
        return idx

    def for_graph(self) -> dict:
        """For matplotlib graph, pie chart"""

        idx = []
        if not len(self.getidx()):
            for _, t in self.getdata():
                if t.startswith(self.sig):
                    t = t[1:].replace(":","").strip()
                    match len(t) <= 20:
                        case True:
                            idx.append(f"{len(idx)+1}-{t}")
                        case False:
                            idx.append(f"{len(idx)+1}-{t[:21]}...")
            return dict(zip(idx, self.getsums(False)))
        else:
            raise ValueError("Data incomplete!")

    def lumpsum(self) -> str:
        """Summing up all total to a string as grand total"""
        
        if tup:= self.getsums(False):
            return f"{sum(tup):,.2f}"

    def sumway(self) -> None:
        """Writing total to data"""

        if idx := self.getidx():
            gsm = self.getsums()
            idx = ((f - s) + s for f, s in zip(gsm.keys(), idx))
            val = iter(gsm.values())
            del gsm
            ind = next(idx)
            lue = next(val)            
            wrt = self.satofi()
            for n, d in self.getdata():
                if ind:
                    if n == ind and d == "\n":
                        wrt.send(f"{self.getchild('child1')*' '}-TOTAL {lue}\n")
                        wrt.send("\n")
                        ind = next(idx, None)
                        lue = next(val, None)
                    else:
                        wrt.send(d)
                else:
                    wrt.send(d)
            else:
                if ind:
                    wrt.send(f"{self.getchild('child1')*' '}-TOTAL {lue}\n")
            wrt.close()
            del idx, val, ind, lue, wrt
        else:
            raise IndexError("No indexes available!")

    def del_total(self) -> None:
        """Delete all Totals"""

        wrt = self.satofi()
        for _, d in self.getdata():
            if not d.strip(" ").startswith("-TOTAL"):
                wrt.send(d)
        wrt.close()
    
    def chksign(self):
        """Checking sign"""

        for n, d in self.getdata():
            if d.startswith(self.sig):
                return True

    def chktot(self):
        """Check Total"""

        for n, d in self.getdata():
            if d.strip(" ").startswith("-TOTAL"):
                return True
                